// Boost.Range library
//
//  Copyright Neil Groves 2009.
//  Use, modification and distribution is subject to the Boost Software
//  License, Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
//  http://www.boost.org/LICENSE_1_0.txt)
//
// For more information, see http://www.boost.org/libs/range/
//
#ifndef BOOST_RANGE_ITERATOR_RANGE_IO_HPP_INCLUDED
#define BOOST_RANGE_ITERATOR_RANGE_IO_HPP_INCLUDED

#include <boost/config.hpp>
#include <boost/detail/workaround.hpp>

#if BOOST_WORKAROUND(BOOST_MSVC, BOOST_TESTED_AT(1500))
    #pragma warning( push )
    #pragma warning( disable : 4996 )
#endif

// From boost/dynamic_bitset.hpp; thanks to Matthias Troyer for Cray X1 patch.
#ifndef BOOST_OLD_IOSTREAMS 
# if defined(__STL_CONFIG_H) && \
    !defined (__STL_USE_NEW_IOSTREAMS) && !defined(__crayx1) \
    /**/
#  define BOOST_OLD_IOSTREAMS
# endif
#endif // #ifndef BOOST_OLD_IOSTREAMS

#ifndef _STLP_NO_IOSTREAMS
# ifndef BOOST_OLD_IOSTREAMS
#  include <ostream>
# else
#  include <ostream.h>
# endif
#endif // _STLP_NO_IOSTREAMS

#include <boost/range/iterator_range_core.hpp>
#include <iterator>
#include <algorithm>
#include <cstddef>

#ifndef BOOST_ITERATOR_RANGE_IO_DELIM
# define BOOST_ITERATOR_RANGE_IO_DELIM ','
#endif

#ifndef BOOST_ITERATOR_RANGE_IO_OPEN
# define BOOST_ITERATOR_RANGE_IO_OPEN '{'
#endif

#ifndef BOOST_ITERATOR_RANGE_IO_CLOSE
# define BOOST_ITERATOR_RANGE_IO_CLOSE '}'
#endif

namespace boost
{
    namespace iterator_range_detail
    {
        template< typename Ostream, typename IteratorT >
        inline void out(Ostream& Os,
                        IteratorT first, IteratorT last)
        {
            bool is_beginning = true;
            for (; first != last; ++first) {
                if (!is_beginning)
                    Os << BOOST_ITERATOR_RANGE_IO_DELIM;
                else
                    is_beginning = false;

                Os << *first;
            }
        }
    } // namespace boost::iterator_range_detail

#ifndef _STLP_NO_IOSTREAMS
# ifndef BOOST_OLD_IOSTREAMS   

        //! iterator_range output operator
        /*!
            Output the range to an ostream. Elements are outputed
            in a sequence without separators.
        */
        template< typename IteratorT, typename Elem, typename Traits >
        inline std::basic_ostream<Elem,Traits>& operator<<( 
                    std::basic_ostream<Elem, Traits>& Os,
                    const iterator_range<IteratorT>& r )
        {
            Os << BOOST_ITERATOR_RANGE_IO_OPEN;
            iterator_range_detail::out(Os, r.begin(), r.end());
            Os << BOOST_ITERATOR_RANGE_IO_CLOSE;
            return Os;
        }

# else

        //! iterator_range output operator
        /*!
            Output the range to an ostream. Elements are outputed
            in a sequence without separators.
        */
        template< typename IteratorT >
        inline std::ostream& operator<<( 
                    std::ostream& Os,
                    const iterator_range<IteratorT>& r )
        {
            Os << BOOST_ITERATOR_RANGE_IO_OPEN;
            iterator_range_detail::out(Os, r.begin(), r.end());
            Os << BOOST_ITERATOR_RANGE_IO_CLOSE;
            return Os;
        }

# endif
#endif // _STLP_NO_IOSTREAMS

} // namespace boost

#undef BOOST_OLD_IOSTREAMS

#if BOOST_WORKAROUND(BOOST_MSVC, BOOST_TESTED_AT(1500))
    #pragma warning(pop)
#endif

#endif // include guard
