// Boost.Range library
//
//  Copyright Akira Takahashi 2010. Use, modification and
//  distribution is subject to the Boost Software License, Version
//  1.0. (See accompanying file LICENSE_1_0.txt or copy at
//  http://www.boost.org/LICENSE_1_0.txt)
//
//
// For more information, see http://www.boost.org/libs/range/
//
#include <boost/range/adaptor/identities.hpp>

#include <boost/test/test_tools.hpp>
#include <boost/test/unit_test.hpp>

#include <boost/assign/list_of.hpp>

#include <sstream>
#include <list>
#include <set>
#include <vector>

namespace boost
{
    namespace
    {
        template< class Container >
        void identities_test_impl()
        {
            using namespace boost::adaptors;

            {
                Container c = boost::assign::list_of(1)(2)(3);
                std::stringstream ss;
                ss << (c | identities);

                BOOST_CHECK_EQUAL(ss.str(), "{1,2,3}");
            }
            {
                const Container c = boost::assign::list_of(1)(2)(3);
                std::stringstream ss;
                ss << (c | identities);

                BOOST_CHECK_EQUAL(ss.str(), "{1,2,3}");
            }
        }

        void identities_test()
        {
            identities_test_impl< std::vector< int > >();
            identities_test_impl< std::list< int > >();
            identities_test_impl< std::set< int > >();
            identities_test_impl< std::multiset< int > >();
        }
    }
}

boost::unit_test::test_suite*
init_unit_test_suite(int argc, char* argv[])
{
    boost::unit_test::test_suite* test
        = BOOST_TEST_SUITE( "RangeTestSuite.adaptor.identities" );

    test->add( BOOST_TEST_CASE( &boost::identities_test ) );

    return test;
}
