// Boost.Range library
//
//  Copyright Akira Takahashi 2010. Use, modification and
//  distribution is subject to the Boost Software License, Version
//  1.0. (See accompanying file LICENSE_1_0.txt or copy at
//  http://www.boost.org/LICENSE_1_0.txt)
//
//
// For more information, see http://www.boost.org/libs/range/
//
#include <boost/range/iterator_range_io.hpp>

#include <boost/test/test_tools.hpp>
#include <boost/test/unit_test.hpp>

#include <boost/assign/list_of.hpp>

#include <string>
#include <sstream>
#include <vector>
#include <list>
#include <set>

template < class Container >
void check_iterator_range_io_impl()
{
    {
        const Container v = boost::assign::list_of(1)(2)(3);

        std::basic_ostringstream<char> ss;
        ss << boost::make_iterator_range(v);
        BOOST_CHECK_EQUAL(ss.str(), "{1,2,3}");
    }
    {
        const Container empty_v;

        std::basic_ostringstream<char> ss;
        ss << boost::make_iterator_range(empty_v);
        BOOST_CHECK_EQUAL(ss.str(), "{}");
    }

    const Container single_v = boost::assign::list_of(3);
    {
        std::basic_ostringstream<char> ss;
        ss << boost::make_iterator_range(single_v);
        BOOST_CHECK_EQUAL(ss.str(), "{3}");
    }
}

void check_iterator_range_io()
{
    check_iterator_range_io_impl< std::vector<int> >();
    check_iterator_range_io_impl< std::list<int> >();
    check_iterator_range_io_impl< std::set<int> >();
    check_iterator_range_io_impl< std::multiset<int> >();
}

boost::unit_test::test_suite* init_unit_test_suite( int argc, char* argv[] )
{
    boost::unit_test::test_suite* test = BOOST_TEST_SUITE( "Range Test Suite" );

    test->add( BOOST_TEST_CASE( &check_iterator_range_io ) );

    return test;
}

